package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.uc.api.model.Scope;
import com.chelvc.cloud.vehicle.api.constant.CategoryType;
import com.chelvc.cloud.vehicle.api.constant.GoodsStatus;
import com.chelvc.cloud.vehicle.api.dto.GoodsDTO;
import com.chelvc.cloud.vehicle.api.dto.GoodsDetailDTO;
import com.chelvc.cloud.vehicle.api.param.CommentQueryParam;
import com.chelvc.cloud.vehicle.api.param.GoodsModifyParam;
import com.chelvc.cloud.vehicle.api.param.GoodsPagingParam;
import com.chelvc.cloud.vehicle.api.param.GoodsQueryParam;
import com.chelvc.cloud.vehicle.server.copier.GoodsCopier;
import com.chelvc.cloud.vehicle.server.dao.GoodsMapper;
import com.chelvc.cloud.vehicle.server.entity.Category;
import com.chelvc.cloud.vehicle.server.entity.Goods;
import com.chelvc.cloud.vehicle.server.service.CategoryService;
import com.chelvc.cloud.vehicle.server.service.CommentService;
import com.chelvc.cloud.vehicle.server.service.CouponService;
import com.chelvc.cloud.vehicle.server.service.GoodsService;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;

import javax.validation.constraints.NotEmpty;
import java.util.List;
import java.util.Objects;

/**
 * 商品业务操作实现
 *
 * @author Woody
 * @date 2023/7/17
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.GoodsService.class)
public class GoodsServiceImpl extends ServiceImpl<GoodsMapper, Goods> implements GoodsService,
        com.chelvc.cloud.vehicle.api.service.GoodsService {
    private final CouponService couponService;
    private final CommentService commentService;
    private final CategoryService categoryService;
    private final MerchantService merchantService;

    @Override
    public List<CategoryType> listMerchantGoodsCategoryTypes(@NonNull Long merchantId) {
        List<CategoryType> types = this.baseMapper.listMerchantGoodsCategoryTypes(merchantId);
        if (!CollectionUtils.isEmpty(types)) {
            types.sort(CategoryType::compareTo);
        }
        return types;
    }

    @Override
    public GoodsDetailDTO getGoodsDetail(@NonNull Long id) {
        Long userId = SessionContextHolder.getId();
        GoodsDetailDTO detail = GoodsDetailDTO.builder().build();
        // 获取商品信息
        Goods goods = AssertUtils.available(this.getById(id), "商品不存在");
        //查询商品是否被收藏
        Integer count = this.baseMapper.goodsFavorite(id,userId);
        detail.setGoods(GoodsCopier.INSTANCE.copying(goods));
        if (count > 0){
            detail.getGoods().setFavorites(true);
        }
        // 获取用户可用优惠券
        if (!CollectionUtils.isEmpty(goods.getCouponIds())) {
            detail.setCoupons(this.couponService.listUserGoodsActiveCoupons(userId, goods.getCouponIds()));
        }

        // 获取商品评价列表
        CommentQueryParam param = CommentQueryParam.builder().size(10).build();
        detail.setComments(this.commentService.listGoodsComments(id, param));
        return detail;
    }

    @Override
    public List<GoodsDTO> listMerchantSimpleGoods(@NonNull Long merchantId, @NonNull GoodsQueryParam param) {
        return this.baseMapper.listMerchantSimpleGoods(merchantId, param);
    }



    @Override
    public Long addGoods(GoodsModifyParam param){
        Scope scope = StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        Long userId = SessionContextHolder.getId();
        Goods goods = GoodsCopier.INSTANCE.copying(param);
        if (scope == Scope.ADMIN) {

        }else{
            List<Long> merchants = this.merchantService.getMerchantIdsByUserId(userId);
            if (ObjectUtils.isEmpty(merchants)) {
                return null;
            }
            goods.setMerchantId(merchants.get(0));
        }
        goods.setScore(0D);
        goods.setSale(0);
        goods.setStatus(GoodsStatus.ONLINE);
        this.save(goods);
        return goods.getId();

    }

    @Override
    public void updateGoods(@NonNull Long id, @NonNull GoodsModifyParam param) {
        Goods goods = AssertUtils.available(this.getById(id), "商品不存在");
        GoodsCopier.INSTANCE.copying(param, goods);
        this.updateById(goods);
    }

    @Override
    public void deleteGoods(@NonNull Long id) {
        AssertUtils.available(this.getById(id), "商品不存在");
        this.baseMapper.deleteById(id);
    }

    @Override
    public Pagination<GoodsDTO> getGoodsPaging(@NonNull GoodsPagingParam param) {
        Long userId = SessionContextHolder.getId();
        Scope scope = StringUtils.ifEmpty(SessionContextHolder.getScope(), Scope::parse);
        if (scope == Scope.ADMIN) {
         }else {
            List<Long> merchants = this.merchantService.getMerchantIdsByUserId(userId);
            if (ObjectUtils.isEmpty(merchants)) {
                return null;
            }
            param.setMerchantId(merchants.get(0));
        }
        Page<Goods> page = this.lambdaQuery()
                .eq(Objects.nonNull(param.getMerchantId()), Goods::getMerchantId, param.getMerchantId())
                .eq(Objects.nonNull(param.getCategoryId()), Goods::getCategoryId, param.getCategoryId())
                .eq(Objects.nonNull(param.getStatus()), Goods::getStatus, param.getStatus())
                .like(StringUtils.notEmpty(param.getName()), Goods::getName, param.getName())
                .orderByDesc(Goods::getCreateTime).page(DatabaseContextHolder.page(param.getPaging()));
        Pagination<GoodsDTO> pagination = DatabaseContextHolder.pagination(page, GoodsCopier.INSTANCE::copying);
        pagination.getRecords().forEach(item ->{
            Category category = this.categoryService.getById(item.getCategoryId());
            if (category != null){
                item.setCategoryName(category.getName());
            }
        });
        return pagination;
    }

    @Override
    public void updateGoodsStatus(@NotEmpty List<Long> ids, @NonNull GoodsStatus status) {
        this.lambdaUpdate().set(Goods::getStatus, status).in(Goods::getId, ids).update();
    }
}
